"""
Toxopy (https://github.com/bchaselab/Toxopy)
© M. Alyetama, University of Nebraska at Omaha
Licensed under the terms of the MIT license
"""

from toxopy import trials, fwarnings
import pandas as pd
from scipy.stats import mannwhitneyu
from os import remove


def calc_dlc_mw(csv_file, export=False):

    df = pd.read_csv(csv_file)

    tls = trials()

    variables = ['vel', 'distance', 'cat_distance', 'acceleration', 'moving']

    for t in tls:

        with open(t, 'w') as f:

            if export is False:

                print('\nTrial:', t, '\n')

                remove(t)

            else:
                print('trial, var, statistics, p', file=f)

            for j in variables:

                pv = df[(df['status'] == 'Positive') & (df['trial'] == t) &
                        (df['var'] == j)]['value']

                nv = df[(df['status'] == 'Negative') & (df['trial'] == t) &
                        (df['var'] == j)]['value']

                stat, p = mannwhitneyu(pv, nv)

                if stat != 0:

                    alpha = 0.05

                    if export is False:

                        print(j, '\nStatistics=%.3f, p=%.3f' % (stat, p))

                        if p > alpha:

                            print('Same distribution (fail to reject H0)\n')
                        elif p < alpha:
                            print('Different distribution (reject H0)\n')

                    else:

                        if p > alpha:

                            print(f'{t},{j},{stat},{p}', file=f)

                        elif p < alpha:
                            print(f'{t},{j},{stat},{p},*', file=f)

    if export is True:

        combined_csv = pd.concat([pd.read_csv(f) for f in tls])

        combined_csv.to_csv("mannwhitneyu_stats_results.csv",
                            index=False,
                            encoding='utf-8-sig')

        [remove(f) for f in tls]



"""
Toxopy (https://github.com/bchaselab/Toxopy)
© M. Alyetama, University of Nebraska at Omaha
Licensed under the terms of the MIT license
"""

import pandas as pd
from pathlib import Path


def improve_dlc_csv(csv_file, trial_type):

    df = pd.read_csv(csv_file)

    df = df.iloc[:, :3]

    n = len(df)

    if trial_type == 'owner':
        time = 1020

    elif trial_type == 'cat':
        time = 600

    f = n / time

    indx, trials = [], []

    for i in range(0, n):
        i = i / f
        indx.append(i)

    result = pd.DataFrame(indx, columns=['time'])

    if trial_type == "owner":

        for i in indx:
            k = 300
            j = 180
            if i < k:
                trials.append('FT')
            elif k < i < k + j:
                trials.append('ST1')
            elif k + j < i < k + j * 2:
                trials.append('UT1')
            elif k + j * 2 < i < k + j * 3:
                trials.append('ST2')
            elif k + j * 3 < i <= k + j * 4:
                trials.append('UT2')

    elif trial_type == "cat":
        t = 120
        for i in indx:
            if i < t:
                trials.append('CA1')
            elif t < i < t * 2:
                trials.append('CA2')
            elif t * 2 < i < t * 3:
                trials.append('CA3')
            elif t * 3 < i < t * 4:
                trials.append('CA4')
            elif t * 4 < i <= t * 5:
                trials.append('CA5')

    tl = pd.DataFrame(trials, columns=['trial'])

    result = pd.concat([df, result, tl], axis=1)

    result = result.iloc[2:, :]

    result.columns = ['indx', 'x', 'y', 'time', 'trial']

    result.to_csv(f'{Path(csv_file).stem}_improved.csv',
                  index=False,
                  sep=',',
                  encoding='utf-8')



"""
Toxopy (https://github.com/bchaselab/Toxopy)
© M. Alyetama, University of Nebraska at Omaha
Licensed under the terms of the MIT license
"""

import pandas as pd
from math import sqrt
from numpy import nan
import os
from tqdm import tqdm
from pathlib import Path
"""Add time and trial type."""


def improve_dlc_output_cat_alone(cat, output_dir):

    if output_dir.endswith('/') is False:

        raise ValueError(
            'Output directory does not end with a trailing slash "/"!')

    def improve_dlc_csv(csv_file):

        df = pd.read_csv(csv_file)
        df = df.iloc[:, :3]

        n = len(df)

        time = 600

        f = n / time

        indx = []

        for i in range(0, n):
            i = i / f
            indx.append(i)

        result = pd.DataFrame(indx, columns=['time'])

        trials = []

        t = 120
        for i in indx:
            if i < t:
                trials.append('Cat alone (1)')
            elif t < i < t * 2:
                trials.append('Cat alone (2)')
            elif t * 2 < i < t * 3:
                trials.append('Cat alone (3)')
            elif t * 3 < i < t * 4:
                trials.append('Cat alone (4)')
            elif t * 4 < i <= t * 5:
                trials.append('Cat alone (5)')

        tl = pd.DataFrame(trials, columns=['trial'])

        result = pd.concat([df, result, tl], axis=1)
        result = result.iloc[2:, :]

        result.columns = ['indx', 'x', 'y', 'time', 'trial']

        result.to_csv(csv_file.strip('.csv') + '_init_improved.csv',
                      index=False,
                      sep=',',
                      encoding='utf-8')

    improve_dlc_csv(cat)
    """Create dataframes for each file type."""

    df_cat = pd.read_csv(cat[:-4] + '_init_improved.csv', sep=",")
    df_cat = pd.DataFrame(df_cat, columns=df_cat.columns)
    df_cat.rename(columns={'x': 'x_cat', 'y': 'y_cat'}, inplace=True)

    tls = pd.DataFrame(df_cat, columns=['trial'])
    """Calculate cat traveled distance."""

    def calculateDistance(x1, y1, x2, y2):
        dist = sqrt((x2 - x1)**2 + (y2 - y1)**2)
        return dist

    df_cat = df_cat.iloc[:, :4]

    cat_dst = []

    n = len(df_cat)

    print("\nCALCULATING THE CAT'S TRAVELED DISTANCE")

    for i in tqdm(range(0, n)):

        x1 = df_cat.iloc[i]['x']
        x2 = df_cat.iloc[i - 1]['x']
        y1 = df_cat.iloc[i]['y']
        y2 = df_cat.iloc[i - 1]['y']

        cat_dst.append(float(calculateDistance(x1, y1, x2, y2)))

    cat_dst[0] = nan

    cat_dst = pd.DataFrame(cat_dst, columns=['cat_distance'])
    """Calculate cat velocity."""

    def calculateVelocity(dist, time):
        return dist / time

    velocity_ls = []

    print("\nCALCULATING THE CAT'S VELOCITY")

    for j in tqdm(range(0, n)):

        if j == 0:
            velocity_ls.append(nan)
            continue

        d = cat_dst.iloc[j]['cat_distance']

        velocity_ls.append(float(calculateVelocity(d, 0.033)))

    velocity = pd.DataFrame(velocity_ls, columns=['velocity'])
    """Calculate cat movement state."""

    moving = []
    notMoving = []

    print("\nESTIMATING THE CAT'S MOVEMENT STATE")

    for z in tqdm(cat_dst['cat_distance']):

        if str(z) == 'nan':
            moving.append(nan)
            notMoving.append(nan)
        elif z >= 0.07:
            moving.append(1)
            notMoving.append(0)
        else:
            moving.append(0)
            notMoving.append(1)

    moving = pd.DataFrame(moving, columns=['moving'])
    notMoving = pd.DataFrame(notMoving, columns=['not_moving'])
    """Calculate cat acceleration."""

    def calculateAcc(vi, vf, ti, tf):
        return (vi - vf) / (ti - tf)

    acc = []

    print("\nCALCULATING THE CAT'S ACCELERATION")

    for q in tqdm(range(0, n)):

        velocity['velocity'].astype('float')

        vi = velocity.iloc[q]['velocity'].astype('float')
        vf = velocity.iloc[q - 1]['velocity'].astype('float')
        ti = df_cat.iloc[q]['time']
        tf = df_cat.iloc[q - 1]['time']

        acc.append(float(calculateAcc(vi, vf, ti, tf)))

    acc = pd.DataFrame(acc, columns=['acceleration'])
    """Clean zero values in relevant data,
    then append data to one csv file."""

    df = pd.concat([df_cat, tls, cat_dst, velocity, moving, notMoving, acc],
                   axis=1)

    cols = ['cat_distance', 'velocity', 'acceleration']

    idx = df.index[df['indx']].tolist()

    print("\nCLEANING OUTPUT")

    for s, w in tqdm(zip(df['velocity'], idx)):
        if s == 0:
            df = df.drop(w)
        elif s > 100:
            df = df.drop(w)

    df[cols] = df[cols].replace({0: nan})

    df.to_csv(output_dir + Path(cat).stem + '_improved.csv',
              index=False,
              sep=',',
              encoding='utf-8')

    os.remove(cat[:-4] + '_init_improved.csv')

    print("\nDONE!")



"""
Toxopy (https://github.com/bchaselab/Toxopy)
© M. Alyetama, University of Nebraska at Omaha
Licensed under the terms of the MIT license
"""

import os
from subprocess import call


def ffconcat(cats, tvp, trial_type):
    # cats is a list of strings ["cat--id"]
    # tvp: /path/to/trials/videos/
    # trial_type: either "cat_alone" or "owner"

    if tvp.endswith('/') is False:

        raise ValueError('Path does not end with a trailing slash "/"')

    for cat in cats:
        tc = tvp + cat
        if trial_type == "cat_alone":
            vid_list = [
                tc + '_T2.mp4', tc + '_T4.mp4', tc + '_T6.mp4', tc + '_T8.mp4',
                tc + '_T10.mp4'
            ]

        elif trial_type == "with_owner":
            vid_list = [
                tc + '_T1.mp4', tc + '_T3.mp4', tc + '_T5.mp4', tc + '_T7.mp4',
                tc + '_T9.mp4'
            ]

        else:
            raise ValueError(
                'False trial type!  The trial type should be either "cat_alone" or "with_owner".'
            )

        cat_txt = f'{cat}.txt'

        if os.path.exists(cat_txt):
            os.remove(cat_txt)

        file_vids = open(cat_txt, 'a')

        for video in vid_list:
            files_list = f'file {video} \n'
            file_vids.write(files_list)
        file_vids.close()

        cat_sh = f'{cat}.sh'

        if os.path.exists(cat_sh):
            os.remove(cat_sh)

        file_sh = open(cat_sh, 'a')

        if trial_type == "cat_alone":

            ffmpeg_command = f'#!/bin/bash\n ffmpeg -f concat -safe 0 -i {cat}.txt -c copy -y {tvp}{cat}_cat.mp4'

        elif trial_type == "with_owner":

            ffmpeg_command = f'#!/bin/bash\n ffmpeg -f concat -safe 0 -i {cat}.txt -c copy -y {tvp}{cat}_owner.mp4'

        file_sh.write(ffmpeg_command)
        file_sh.close()

        os.chmod('./' + cat + '.sh', 0o755)
        call('./' + cat + '.sh', shell=True)



"""
Toxopy (https://github.com/bchaselab/Toxopy)
© M. Alyetama, University of Nebraska at Omaha
Licensed under the terms of the MIT license
"""

from toxopy import trials
import json
from numpy import mean, median


def obtain_grand_m(json_file_loc):

    # json_file_loc is the path to the json file with all individual avgs data
    # output_dir is where you want to dump the .csv file to (optional)

    with open(json_file_loc) as json_file:
        data = json.load(json_file)

    var = ['distance', 'cat_distance', 'vel', 'acceleration', 'moving']

    tls = trials()

    dct = {}

    for i in var:
        dct['p%s' % i] = []
        dct['n%s' % i] = []

    for p, n in zip(list(data['positive'].keys()),
                    list(data['negative'].keys())):

        for t in tls:

            for v, o in zip(var, range(0, 5)):

                if v == var[o]:
                    dct['p' + var[o]].append(data['positive'][p][t][v])
                    dct['n' + var[o]].append(data['negative'][n][t][v])

    for y in ['pdistance', 'ndistance']:
        dct[y] = [i for i in dct[y] if i != "NaN"]

    with open('grand_avgs_mds.csv', 'w') as f:

        print('infection_status,variable,mean,median', file=f)

        for g in dct:

            for q in ['positive', 'negative']:

                if g.startswith(q[0]):

                    print(f'{q},{g[1:]},{mean(dct[g])},{median(dct[g])}',
                          file=f)



"""
Toxopy (https://github.com/bchaselab/Toxopy)
© M. Alyetama, University of Nebraska at Omaha
Licensed under the terms of the MIT license
"""

import pandas as pd
from math import sqrt
from numpy import nan
from os import remove
from tqdm import tqdm
from rich.console import Console


def improve_dlc_output(cat, owner):

    console = Console()

    def improve_dlc_csv(csv_file, trial_type):
        """Add time and trial type."""

        df = pd.read_csv(csv_file)
        df = df.iloc[:, :3]

        n = len(df)

        if trial_type == 'owner':
            time = 1020

        elif trial_type == 'cat':
            time = 600

        f = n / time

        indx = []

        for i in range(0, n):
            i = i / f
            indx.append(i)

        result = pd.DataFrame(indx, columns=['time'])

        trials = []

        if trial_type == "owner":

            for i in indx:
                k = 300
                j = 180
                if i < k:
                    trials.append('FT')
                elif k < i < k + j:
                    trials.append('ST1')
                elif k + j < i < k + j * 2:
                    trials.append('UT1')
                elif k + j * 2 < i < k + j * 3:
                    trials.append('ST2')
                elif k + j * 3 < i <= k + j * 4:
                    trials.append('UT2')

        elif trial_type == "cat":
            t = 120
            for i in indx:
                if i < t:
                    trials.append('CA1')
                elif t < i < t * 2:
                    trials.append('CA2')
                elif t * 2 < i < t * 3:
                    trials.append('CA3')
                elif t * 3 < i < t * 4:
                    trials.append('CA4')
                elif t * 4 < i <= t * 5:
                    trials.append('CA5')

        tl = pd.DataFrame(trials, columns=['trial'])

        result = pd.concat([df, result, tl], axis=1)
        result = result.iloc[2:, :]

        result.columns = ['indx', 'x', 'y', 'time', 'trial']

        result.to_csv(csv_file.strip('.csv') + '_improved.csv',
                      index=False,
                      sep=',',
                      encoding='utf-8')

    improve_dlc_csv(cat, trial_type='owner')
    improve_dlc_csv(owner, trial_type='owner')
    """Calculate distance between cat and owner."""

    cat_improved = cat.strip('.csv') + '_improved.csv'
    df_cat = pd.read_csv(cat_improved, sep=",")
    df_cat = pd.DataFrame(df_cat, columns=df_cat.columns)

    owner_improved = owner.strip('.csv') + '_improved.csv'
    df_owner = pd.read_csv(owner_improved, sep=",")
    df_owner = pd.DataFrame(df_owner, columns=df_owner.columns)

    ds = []

    console.print(
        "\nCALCULATING THE DISTANCE BETWEEN CAT AND OWNER", style="bold green")

    for i, j in tqdm(zip(df_cat['indx'], df_owner['indx'])):

        p1 = [df_cat['x'][i], df_cat['y'][i]]
        p2 = [df_owner['x'][j], df_owner['y'][j]]

        distance = sqrt(((p1[0] - p2[0])**2) + ((p1[1] - p2[1])**2))

        ds.append(distance)

    ds = pd.DataFrame(ds, columns=['distance'])

    df_cat.rename(columns={'x': 'x_cat', 'y': 'y_cat'}, inplace=True)
    df_owner.rename(columns={'x': 'x_owner', 'y': 'y_owner'}, inplace=True)

    result = pd.concat([
        df_cat[{'indx', 'x_cat', 'y_cat'}],
        df_owner[{'x_owner', 'y_owner', 'time', 'trial'}], ds
    ],
        axis=1)

    result = result[[
        'indx', 'time', 'trial', 'x_cat', 'y_cat', 'x_owner', 'y_owner',
        'distance'
    ]]

    cols = ["x_cat", "y_cat", "x_owner", "y_owner"]

    result[cols] = result[cols].replace({0: nan})
    """Calculate cat traveled distance."""

    def calculateDistance(x1, y1, x2, y2):
        dist = sqrt((x2 - x1)**2 + (y2 - y1)**2)
        return dist

    df_cat = df_cat.iloc[:, :4]

    cat_dst = []

    n = len(df_cat)

    i = 0

    console.print("\nCALCULATING THE CAT'S TRAVELED DISTANCE",
                  style="bold green")

    for i in tqdm(range(0, n)):

        x1 = df_cat.iloc[i]['x_cat']
        x2 = df_cat.iloc[i - 1]['x_cat']
        y1 = df_cat.iloc[i]['y_cat']
        y2 = df_cat.iloc[i - 1]['y_cat']

        cat_dst.append(float(calculateDistance(x1, y1, x2, y2)))

    cat_dst[0] = nan

    cat_dst = pd.DataFrame(cat_dst, columns=['cat_distance'])
    """Calculate cat velocity."""

    def calculateVelocity(dist, time):
        return dist / time

    velocity_ls = []

    console.print("\nCALCULATING THE CAT'S VELOCITY", style="bold green")

    for j in tqdm(range(0, n)):

        if j == 0:
            velocity_ls.append(nan)
            continue

        d = cat_dst.iloc[j]['cat_distance']

        # t = df_cat.iloc[j]['time'] - df_cat.iloc[j - 1]['time']

        velocity_ls.append(float(calculateVelocity(d, 0.033)))

    velocity = pd.DataFrame(velocity_ls, columns=['velocity'])
    """Calculate cat movement state."""

    moving = []
    notMoving = []

    console.print("\nESTIMATING THE CAT'S MOVEMENT STATE", style="bold green")

    for z in tqdm(cat_dst['cat_distance']):

        if str(z) == 'nan':
            moving.append(nan)
            notMoving.append(nan)
        elif z >= 0.07:
            moving.append(1)
            notMoving.append(0)
        else:
            moving.append(0)
            notMoving.append(1)

    moving = pd.DataFrame(moving, columns=['moving'])
    notMoving = pd.DataFrame(notMoving, columns=['not_moving'])
    """Calculate cat acceleration."""

    def calculateAcc(vi, vf, ti, tf):
        return (vi - vf) / (ti - tf)

    acc = []

    console.print("\nCALCULATING THE CAT'S ACCELERATION", style="bold green")

    for q in tqdm(range(0, n)):

        velocity['velocity'].astype('float')

        vi = velocity.iloc[q]['velocity'].astype('float')
        vf = velocity.iloc[q - 1]['velocity'].astype('float')
        ti = df_cat.iloc[q]['time']
        tf = df_cat.iloc[q - 1]['time']

        acc.append(float(calculateAcc(vi, vf, ti, tf)))

    acc = pd.DataFrame(acc, columns=['acceleration'])
    """Clean zero values in relevant data,
    then append data to one csv file."""

    df = pd.concat([result, cat_dst, velocity, moving, notMoving, acc], axis=1)

    cols = ['cat_distance', 'velocity', 'acceleration']

    idx = df.index[df['indx']].tolist()

    console.print("\nCLEANING OUTPUT", style="bold green")

    for s, w in tqdm(zip(df['velocity'], idx)):
        if s == 0:
            df = df.drop(w)
        elif s > 100:
            df = df.drop(w)

    df[cols] = df[cols].replace({0: nan})

    [remove(i) for i in [owner_improved, cat_improved]]

    df.to_csv(cat.strip('chwo.csv') + 'improved.csv',
              index=False,
              sep=',',
              encoding='utf-8')

    console.print("\nDONE!", style="bold green")



"""Object-specific python package to run automated tasks in the Chase Lab."""
from .recycle import *

from .ffsync import ffsync
from .ffconcat import ffconcat
from .improve_dlc_output import improve_dlc_output
from .improve_dlc_csv import improve_dlc_csv
from .improve_dlc_output_cat_alone import improve_dlc_output_cat_alone
from .csv2h5 import csv2h5
from .analyze_rois import analyze_rois
from .concat_rois import concat_rois
from .roi_calc_mw import roi_calc_mw
from .json2tidycsv import json2tidycsv
from .obtain_grand_m import obtain_grand_m
from .concat_csv import concat_csv
from .calc_dlc_mw import calc_dlc_mw
from .dlcboxplot import dlcboxplot
from .fwarnings import fwarnings
from .one_cat_one_file import one_cat_one_file
from .correct_times import correct_times
from .analyze_turnpoints import analyze_turnpoints
from .return_sem import return_sem

from .version import __version__, VERSION



"""
Toxopy (https://github.com/bchaselab/Toxopy)
© M. Alyetama, University of Nebraska at Omaha
Licensed under the terms of the MIT license
"""

from toxopy import fwarnings
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt


def dlcboxplot(file,
               variable,
               ylab,
               jitter=False,
               colors=False,
               title=False,
               save=False,
               output_dir=False):

    df = pd.read_csv(file)
    df = df[df['var'] == variable]

    sns.set(style="ticks", font_scale=1)

    plt.figure(figsize=(13, 5), dpi=100)

    if colors is False:
        my_pal = {"Negative": "#00FFFF", "Positive": "#E60E3C"}
    else:
        my_pal = {"Negative": colors[0], "Positive": colors[1]}

    ax = sns.boxplot(x='trial',
                     y='value',
                     data=df,
                     hue='status',
                     palette=my_pal)

    if jitter is True:

        sns.stripplot(x='trial',
                      y='value',
                      data=df,
                      color='black',
                      size=3,
                      jitter=1)
    '''add vertical lines to seperate boxplots pairs (style)'''
    for i in range(len(df['trial'].unique()) - 1):
        if variable == 'vel':
            plt.vlines(i + .5,
                       10,
                       45,
                       linestyles='solid',
                       colors='black',
                       alpha=0.2)
        elif variable == 'cat_distance':
            plt.vlines(i + .5,
                       0,
                       1.3,
                       linestyles='solid',
                       colors='black',
                       alpha=0.2)

    if title is not False:
        plt.title(title, fontsize=14)
    else:
        pass

    ax.set(xlabel='Trial', ylabel=ylab)

    plt.legend(title='Infection Status')
    '''add significance bars and asterisks between boxes'''
    # [first pair, second pair], ..., [|, –], ...
    if variable == 'vel':
        l = [[7.75, 5.75], [8.25, 6.25], [26, 28], [31, 33]]
    elif variable == 'cat_distance':
        l = [[7.75, 5.75], [8.25, 6.25], [0.85, 0.9], [0.95, 1]]

    for x1, x2, y1, y2 in zip(l[0], l[1], l[2], l[3]):
        sig = plt.plot([x1, x1, x2, x2], [y1, y2, y2, y1],
                       linewidth=1,
                       color='k')
        plt.text((x1 + x2) * .5, y2 + 0, "*", ha='center', va='bottom')

    plt.show()

    if save is True:
        fig = ax.get_figure()
        if output_dir is not False:
            fig.savefig(f'{output_dir}/{variable}.png',
                        bbox_inches="tight",
                        dpi=100,
                        pad_inches=0.1)
        else:
            fig.savefig(f'{variable}.png',
                        bbox_inches="tight",
                        dpi=100,
                        pad_inches=0.1)



"""
Toxopy (https://github.com/bchaselab/Toxopy)
© M. Alyetama, University of Nebraska at Omaha
Licensed under the terms of the MIT license
"""

import pandas as pd
from numpy import mean
import json
from toxopy import trials


def jsonify_dlc_avgs(csv_file):

    # csv_file is a .csv file with all the data from all the experiment trials

    df = pd.read_csv(csv_file)

    def percentage(part, whole):
        return 100 * float(part) / float(whole)

    cats = df.cat.unique()

    tls = trials()

    d = {}

    for cat in cats:

        df2 = df.loc[(df['cat'] == cat)]
        d[cat] = {}

        for t in tls:
            df3 = df2.loc[(df['trial'] == t)]
            d[cat][t] = {}
            d[cat][t]['distance'] = mean(df3['distance_loess05'])
            d[cat][t]['cat_distance'] = mean(df3['cat_distance_loess05'])
            d[cat][t]['vel'] = mean(df3['velocity_loess05'])
            d[cat][t]['acceleration'] = mean(df3['acceleration_loess05'])
            d[cat][t]['moving'] = percentage(sum(df3['moving']), len(df2))

    with open('dlc_avgs.json', 'w') as outfile:
        json.dump(d, outfile)



"""
Toxopy (https://github.com/bchaselab/Toxopy)
© M. Alyetama, University of Nebraska at Omaha
Licensed under the terms of the MIT license
"""

import pandas as pd
from pathlib import Path
import glob
from toxopy import trials_cap, trials
from os import remove
from tqdm import tqdm


def correct_times(csv_dir, output_dir):
    """'csv_dir' is the path to the folder containing the combined csv files"""

    files = glob.glob(f'{csv_dir}/*.csv')

    tls = trials_cap()

    trials_times = [300, 420, 600, 720, 900, 1020, 1200, 1320, 1500, 1620]

    additions = [0, 300, 120, 480, 240, 660, 360, 840, 480, 1020]

    for file in tqdm(files):

        df = pd.read_csv(file, header=[0])

        cat = Path(file).stem

        for trial, ttime, ad in zip(tls, trials_times, additions):

            time = df.loc[(df['trial'] == trial)]['time']

            diff = max(time) - min(time)

            if trial == trial and diff > ttime:

                raise ValueError('Fails!', cat, trial, diff)

            elif trial == trial and diff < ttime:

                def subdf(variable):

                    return df[df['trial'] == trial][variable]

                t, v, a, r = subdf('time') + ad, subdf(
                    'velocity_loess05'), subdf('acceleration_loess05'), subdf(
                        'trial')

                fdf = pd.DataFrame([t, v, a, r]).T

                fdf.to_csv(f'{output_dir}/{cat}_{trial}.csv',
                           index=False,
                           encoding='utf-8-sig')

        fs = glob.glob(f'{output_dir}/{cat}_*.csv')

        ccsv = pd.concat([pd.read_csv(i) for i in fs])

        old_names, new_names = trials_cap(), trials()

        ccsv['trial'] = ccsv['trial'].replace(old_names, new_names)

        ccsv = ccsv.sort_values(by=['time'])

        ccsv.to_csv(f'{output_dir}/{cat}.csv',
                    index=False,
                    encoding='utf-8-sig')

        [remove(i) for i in fs]



"""
Toxopy (https://github.com/bchaselab/Toxopy)
© M. Alyetama, University of Nebraska at Omaha
Licensed under the terms of the MIT license
"""

import json
from toxopy import trials
from os import remove


def json2tidycsv(json_file_loc, csv_output=False):

    # Converts dlc avgs data from JSON format to a tidy data in csv
    # json_file_loc is the path to the json file with all individual avgs data

    with open(json_file_loc) as json_file:
        data = json.load(json_file)

    param = ['distance', 'vel', 'cat_distance', 'acceleration', 'moving']

    tls = trials()

    with open('dlc_avgs_data_tidy.csv', 'w') as f:

        for i in ['positive', 'negative']:

            for c in list(data[i].keys()):

                for x in tls:

                    for z in param:

                        if csv_output is True:

                            print(f'{c},positive,{z},{x},{data[i][c][x][z]}',
                                  file=f)

                        else:

                            print(f'{c},positive,{z},{x},{data[i][c][x][z]}')

        remove('dlc_avgs_data_tidy.csv') if csv_output is False else None



"""
Toxopy (https://github.com/bchaselab/Toxopy)
© M. Alyetama, University of Nebraska at Omaha
Licensed under the terms of the MIT license
"""

import pandas as pd
from glob import glob
from scipy.stats import sem
from numpy import mean


def return_sem(directory):
    """Directory is the path to all 'evaluation results' files"""

    def calc_sem():

        files = glob(f'{directory}/*.csv')

        train_err, test_err = [], []

        for file in files:

            df = pd.read_csv(file)

            train_err.append(float(df[' Train error(px)']))
            test_err.append(float(df[' Test error(px)']))

        return [[mean(train_err), mean(test_err)], [sem(train_err), sem(test_err)]]

    def results(i):
        return [round(x, 4) for x in calc_sem()[i]]

    return {'Train Error': {'Mean': results(0)[0], 'S.E.M.': results(1)[0]},
            'Test Error': {'Mean': results(0)[1], 'S.E.M.': results(1)[1]}}



"""
Toxopy (https://github.com/bchaselab/Toxopy)
© M. Alyetama, University of Nebraska at Omaha
Licensed under the terms of the MIT license
"""

import pandas as pd
from pathlib import Path
import glob
import os
from toxopy import trials, concat_csv, set_status


def analyze_turnpoints(improved_dlc_dir, init_tp_dir, output_dir):
    """
    'improved_dlc_dir' the path to improved_dlc files (timed & combined)
    'init_tp_dir' is the initial turnpoints csv files dir (R output)
    'output_dir' is the dir in which the output files will be saved
    """

    def improve_turnpoints(init_tp_dir, output_dir):
        """
        'init_tp_dir' is the initial turnpoints csv files dir
        'output_dir' is the dir in which the output files will be saved
        """

        tt = [300, 420, 600, 720, 900, 1020, 1200, 1320, 1500, 1620]

        files = glob.glob(f'{init_tp_dir}/*.csv')

        for file in files:

            df = pd.read_csv(file)

            tls, times, proba = [], [], []

            trls = trials()

            for x in df['tppos']:
                times.append(round(x / 30, 4))
            for y in df['proba']:
                proba.append(format(y, '.8f'))

            df = df.drop(columns=['proba'])

            k, j, t = 300, 180, 120

            for i in times:
                if i < tt[0]:
                    tls.append('FT')
                else:
                    for q, p in zip(range(0, 10), trls[1:]):
                        if tt[q] < i < tt[q+1]:
                            tls.append(p)

            df['proba'], df['trial'], df['time'] = proba, tls, times

            df.to_csv(f'{output_dir}/{Path(file).stem[:-11]}.csv',
                      index=False,
                      sep=',',
                      encoding='utf-8')

    improve_turnpoints(init_tp_dir, output_dir)

    def find_tps_velocity_values(improved_dlc_dir, output_dir, super_output_dir):
        """
        'improved_dlc_dir' is the directory with the 'improved_dlc' files
        'output_dir' is the directory with the improved turning points csv files
        'super_output_dir' is the directory where output files will be save
        """
        def tp_csv_file(cat):

            fs = glob.glob(f'{output_dir}/{cat}.csv')

            for f in fs:
                return pd.read_csv(f)

        files = glob.glob(f'{improved_dlc_dir}/*.csv')

        for file in files:

            df = pd.read_csv(file)

            cat = Path(file).stem

            df2 = tp_csv_file(cat)

            velocity_value = []

            for i in df2['tppos']:
                velocity_value.append(df['velocity_loess05'][i])

            df2['velocity_value'] = velocity_value

            df2.to_csv(f'{super_output_dir}/{Path(file).stem}.csv',
                       index=False,
                       encoding='utf-8-sig')

    find_tps_velocity_values(improved_dlc_dir, output_dir, output_dir)

    if not os.path.exists(f'{output_dir}/diff'):
        os.makedirs(f'{output_dir}/diff')

    diff_output_dir = f'{output_dir}/diff'

    def find_turnpoints_diff(output_dir_super, diff_output_dir):
        """
        'output_dir_super' is 'turnpoints_super_improved' dir
        'output_dir_diff' is the directory in which the output files will be saved
        """

        for variable in ['time', 'velocity_value']:

            files = glob.glob(f'{output_dir_super}/*.csv')

            tls = trials()

            for file in files:

                df = pd.read_csv(file)

                if variable == 'time':
                    firstispeak = str(df['firstispeak'][0])

                cat = Path(file).stem

                def altE(l):
                    if firstispeak == 'False':
                        return l[::2]

                    elif firstispeak == 'True':
                        return l[1::2]

                ls = df[variable].tolist()

                diff, trial, diff_calc = [], [], []

                if variable == 'time':
                    n = len(altE(ls))

                elif variable == 'velocity_value':
                    n = len(ls[::2])

                for i in range(0, n):

                    if i == n - 1:
                        break

                    else:
                        if variable == 'time':
                            diff.append(altE(ls)[i + 1] - altE(ls)[i])
                            trial.append(altE(df['trial'].tolist()))
                            diff_calc.append(
                                f'({altE(ls)[i + 1]}) - ({altE(ls)[i]})')

                        elif variable == 'velocity_value':
                            diff.append(ls[1::2][i] - ls[::2][i])
                            tl = df['trial'].tolist()
                            trial.append(tl[::2])
                            diff_calc.append(
                                f'({round(ls[1::2][i], 4)}) - ({round(ls[::2][i], 4)})')

                diff = [round(x, 4) for x in diff]

                df = pd.DataFrame()

                if variable == 'time':
                    df['time_diff'] = diff
                    df['time_diff'] = df['time_diff'].abs()

                elif variable == 'velocity_value':
                    df['vel_diff'] = diff
                    df['vel_diff'] = df['vel_diff'].abs()

                df['cat'] = cat
                set_status(cat, df)
                df['trial'] = trial[0][0:len(diff)]
                df['diff_calc'] = diff_calc

                df = df.reset_index()

                for i, j, z in zip(df['trial'], df['trial'][1:], df['index']):
                    if i != j:
                        df.drop(z, inplace=True)

                df.to_csv(f'{diff_output_dir}/{cat}.csv',
                          index=False, encoding='utf-8-sig')

            if variable == 'velocity_value':
                concat_csv(diff_output_dir, "all_vel_diff")

            elif variable == 'time':
                concat_csv(diff_output_dir, "all_time_diff")

    find_turnpoints_diff(output_dir, diff_output_dir)



"""
Toxopy (https://github.com/bchaselab/Toxopy)
© M. Alyetama, University of Nebraska at Omaha
Licensed under the terms of the MIT license
"""

import pandas as pd
from glob import glob
from pathlib import Path


def one_cat_one_file(dir1, dir2, output_dir):
    """'dir1' is the dir with cat_alone_improved csv files
        'dir2' is the dir with with_owner_improved csv files"""

    cat_alone, with_owner = glob(dir1 + '/*.csv'), glob(dir2 + '/*.csv')

    for f1, f2 in zip(sorted(cat_alone), sorted(with_owner)):

        cat1, cat2 = Path(f1).stem[:-10], Path(f2).stem[:-11]

        files = []

        if cat1 == cat2:
            files.append(f1)
            files.append(f2)

        else:
            raise ValueError('Something is wrong!')

        combined_csv = pd.concat([
            pd.read_csv(f,
                        usecols=[
                            'time', 'velocity_loess05', 'acceleration_loess05',
                            'trial'
                        ]) for f in files
        ])

        combined_csv.to_csv(f'{output_dir}/{cat1}.csv',
                            index=False,
                            encoding='utf-8-sig')



"""
Toxopy (https://github.com/bchaselab/Toxopy)
© M. Alyetama, University of Nebraska at Omaha
Licensed under the terms of the MIT license
"""

import pandas as pd
import glob


def concat_csv(directory, output_file_name):

    files = glob.glob(directory + '/*.csv')

    combined_csv = pd.concat([pd.read_csv(f) for f in files])

    combined_csv.to_csv(directory + '/' + output_file_name + '.csv',
                        index=False,
                        encoding='utf-8-sig')



"""
Toxopy (https://github.com/bchaselab/Toxopy)
© M. Alyetama, University of Nebraska at Omaha
Licensed under the terms of the MIT license
"""


def trials_full():

    return [
        'no treatment', 'cat alone (1)', 'first saline', 'cat alone (2)',
        'first urine', 'cat alone (3)', 'second saline', 'cat alone (4)',
        'second urine', 'cat alone (5)'
    ]


def trials_cap():

    return [
        'No treatment', 'Cat alone (1)', 'First Saline', 'Cat alone (2)',
        'First Urine', 'Cat alone (3)', 'Second Saline', 'Cat alone (4)',
        'Second Urine', 'Cat alone (5)'
    ]


def trials():

    return [
        'FT', 'CA1', 'ST1', 'CA2', 'UT1', 'CA3', 'ST2', 'CA4', 'UT2', 'CA5'
    ]


def set_status(cat, df):

    if cat in ['daisy', 'mila', 'marmalade', 'washburne', 'lavoisier']:

        df['infection_status'] = 'Positive'

    else:

        df['infection_status'] = 'Negative'



"""
Toxopy (https://github.com/bchaselab/Toxopy)
© M. Alyetama, University of Nebraska at Omaha
Licensed under the terms of the MIT license
"""

import warnings


def fwarnings():
    warnings.filterwarnings("ignore")


fwarnings()



"""
Toxopy (https://github.com/bchaselab/Toxopy)
© M. Alyetama, University of Nebraska at Omaha
Licensed under the terms of the MIT license
"""

import os
import glob
from toxopy import fwarnings
import matplotlib.patches as patches
from collections import namedtuple
from dlcu import time_in_each_roi
import matplotlib.pyplot as plt
import json
import numpy as np
from pathlib import Path
import pandas as pd

# %matplotlib inline


def analyze_rois(file, room_layout, output_dir, trial_type, span=10):

    if output_dir.endswith('/') is False:

        raise ValueError(
            'Output directory does not end with a trailing slash "/"!')
    pass

    # Load dataset

    df = pd.read_csv(file)
    df = df.dropna()

    if trial_type == 'with_owner':

        t1 = df.loc[df['trial'] == 'FT']
        t3 = df.loc[df['trial'] == 'ST1']
        t5 = df.loc[df['trial'] == 'UT1']
        t7 = df.loc[df['trial'] == 'ST2']
        t9 = df.loc[df['trial'] == 'UT2']

        cat = str(Path(file).stem)[:-11]

        trials = [t1, t3, t5, t7, t9]

        trials_names = ['FT', 'ST1', 'UT1', 'ST2', 'UT2']

    elif trial_type == 'cat_alone':

        t2 = df.loc[df['trial'] == 'CA1']
        t4 = df.loc[df['trial'] == 'CA2']
        t6 = df.loc[df['trial'] == 'CA3']
        t8 = df.loc[df['trial'] == 'CA4']
        t10 = df.loc[df['trial'] == 'CA5']

        cat = str(Path(file).stem)[:-10]

        trials = [t2, t4, t6, t8, t10]

        trials_names = ['CA1', 'CA2', 'CA3', 'CA4', 'CA5']

    # Create variable names

    for trial, trial_name in zip(trials, trials_names):

        # time = trial['time']
        span = str(span)

        velocity = trial['velocity_loess' + span]

        if trial_type == 'with_owner':

            x_cat = trial['x_cat_loess' + span]
            y_cat = trial['y_cat_loess' + span]

        elif trial_type == 'cat_alone':

            x_cat = trial['x_loess' + span]
            y_cat = trial['y_loess' + span]

            # Calculate and plot rois

            # def rois_gen(i):
            #     def getList(dct):
            #         lst = []
            #         for key in dct.keys():
            #             lst.append(key)

            #         return lst

            with open(room_layout) as json_file:
                p = json.load(json_file)

            # catslist = getList(p)

            position = namedtuple('position', ['topleft', 'bottomright'])

            # two points defining each roi: topleft(X,Y) and bottomright(X,Y).

            return {
                'middle':
                position((p[i]['middle_tl'][0], p[i]['middle_tl'][1]),
                         (p[i]['middle_br'][0], p[i]['middle_br'][1])),
                'rightside':
                position((p[i]['right_tl'][0], p[i]['right_tl'][1]),
                         (p[i]['right_br'][0], p[i]['right_br'][1])),
                'leftside':
                position((p[i]['left_tl'][0], p[i]['left_tl'][1]),
                         (p[i]['left_br'][0], p[i]['left_br'][1])),
                'topside':
                position((p[i]['top_tl'][0], p[i]['top_tl'][1]),
                         (p[i]['top_br'][0], p[i]['top_br'][1])),
                'bottomside':
                position((p[i]['bottom_tl'][0], p[i]['bottom_tl'][1]),
                         (p[i]['bottom_br'][0], p[i]['bottom_br'][1]))
            }

        bp_tracking = np.array((x_cat, y_cat, velocity))

        rois = rois_gen(cat)

        fig, ax = plt.subplots(1)

        plt.plot(x_cat, y_cat, '.-')

        positions = [
            'middle', 'rightside', 'leftside', 'topside', 'bottomside'
        ]
        colors = ['orange', 'purple', 'red', 'blue', 'green']

        for position, color in zip(positions, colors):

            rect = patches.Rectangle(
                rois[position].topleft,
                rois[position].bottomright[0] - rois[position].topleft[0],
                rois[position].bottomright[1] - rois[position].topleft[1],
                linewidth=1,
                edgecolor=color,
                facecolor='none')
            ax.add_patch(rect)

        plt.ylim(-100, 600)
        # plt.show()
        plt.close()

        res = time_in_each_roi.get_timeinrois_stats(bp_tracking.T,
                                                    rois,
                                                    fps=30,
                                                    returndf=True,
                                                    check_inroi=True)

        current_trial = []

        # for x in range(0, len(res)):

        #     current_trial.append(trial_name)

        res['trial'] = current_trial

        walls = res.loc[(res['ROI_name'] == 'rightside') |
                        (res['ROI_name'] == 'leftside') |
                        (res['ROI_name'] == 'topside') |
                        (res['ROI_name'] == 'bottomside')]

        middle = res.loc[res['ROI_name'] == 'middle']

        df_walls = walls.sum(axis=0)
        df_walls['ROI_name'] = 'walls'
        df_walls['trial'] = trial_name
        df_walls = df_walls.T

        df_middle = middle.sum(axis=0)
        df_middle['ROI_name'] = 'middle'
        df_middle['trial'] = trial_name
        df_middle = df_middle.T

        df_final = pd.concat([df_walls, df_middle], axis=1, sort=False)
        df_final = df_final.T

        df_final.to_csv(output_dir + cat + trial_name + '_deleteme' + '.csv',
                        index=False,
                        sep=',',
                        encoding='utf-8')

    files = glob.glob(output_dir + '*_deleteme.csv')

    for file in files:

        df2 = pd.read_csv(file)

        df2 = df2[[
            'ROI_name', 'transitions_per_roi', 'cumulative_time_in_roi_sec',
            'avg_time_in_roi_sec', 'avg_vel_in_roi', 'trial'
        ]]

        df2.to_csv(file, index=False)

    combined_csv = pd.concat([pd.read_csv(f) for f in files])

    if trial_type == 'with_owner':

        sort_by_trial = {'FT': 0, 'ST1': 1, 'UT1': 3, 'ST2': 4, 'UT2': 5}

    elif trial_type == 'cat_alone':

        sort_by_trial = {'CA1': 0, 'CA2': 1, 'CA3': 3, 'CA4': 4, 'CA5': 5}

    combined_csv = combined_csv.iloc[combined_csv['trial'].map(
        sort_by_trial).argsort()]

    if trial_type == 'with_owner':

        combined_csv.to_csv(output_dir + cat + "_with_owner_rois.csv",
                            index=False,
                            encoding='utf-8-sig')

    elif trial_type == 'cat_alone':

        combined_csv.to_csv(output_dir + cat + "_cat_alone_rois.csv",
                            index=False,
                            encoding='utf-8-sig')

    [os.remove(f) for f in files]



"""
Toxopy (https://github.com/bchaselab/Toxopy)
© M. Alyetama, University of Nebraska at Omaha
Licensed under the terms of the MIT license
"""

import pandas as pd
from tqdm import tqdm
import glob
from pathlib import Path


def csv2h5(path, head=[1]):
    def toh5(file):
        df = pd.read_csv(file, header=head, index_col=[0])

        return df.to_hdf(Path(file).stem + '.h5',
                         'data',
                         mode='w',
                         format='table')

    if isinstance(path, str):

        csv_files_dir = path + '/*.csv'

        files = glob.glob(csv_files_dir)

        for file in tqdm(files):

            toh5(file)

    elif isinstance(path, list):

        for file in tqdm(path):

            toh5(file)
